import java.util.ArrayList;

public class ArrayBag<T> implements BagInterface<T>
{
    private ArrayList<T> list; 
    private static final int DEFAULT_CAPACITY = 25; // reserved word "final" means "constant"
    private int mySize=0;
    private int myCapacity;

    /** Creates an empty bag whose initial capacity is 25. */
    public ArrayBag() 
    {
        this(DEFAULT_CAPACITY);// calling explicit constructor with default capacity
        //myArray = new Object[DEFAULT_CAPACITY]; // classic approach
    } // end default constructor

    /** Creates an empty bag having a given initial capacity.
        @param capacity  the integer capacity desired */
    public ArrayBag(int capacity) 
    {
        list = new ArrayList<T>(capacity);
        myCapacity = capacity;
    } // end constructor

    /** Adds a new entry to this bag.
        @param newEntry  the object to be added as a new entry
        @return true if the addition is successful, or false if not */
    public boolean add(T newEntry) 
    {
        boolean result = true;
        // check if the bag is empty before inserting item
        if(isFull()){
            result =  false; // bag is full
        }else{
            list.add(newEntry); // add new entry to the internal array
       }
    
        return result; // return addition confirmation
    } // end add

    /** Sees whether this bag is empty.
        @return true if this bag is empty, or false if not */
    public boolean isEmpty() 
    {
        return list.size() == 0;
    } // end isEmpty
    
    /** Sees whether this bag is full.
       @return true if this bag is full, or false if not */
    public boolean isFull() 
    {
        return list.size() == myCapacity;        
    } // end isFull

    /** Gets the current number of objects in this bag.
         @return the integer number of objects currently in the bag */
    public int getCurrentSize() 
    {
        return list.size();
    } // end getCurrentSize

    /** Counts the number of times a given entry appears in this bag.
         @param anEntry  the entry to be counted
         @return the number of times anEntry appears in the bag */
    public int getFrequencyOf(T anEntry) 
    {
        int numTimes = 0; // number of occurrences
        // loop over all the items in the array up seaching coincidences
        for(int k=0; k < list.size(); k++){
            // check if the Object anEntry is inside the myArray
            if(list.get(k).equals(anEntry)){
                // count all occurrences
                numTimes++;
            }
        }
    
        return numTimes; // return number of ocurrences
    } // end getFrequencyOf

    /** Tests whether this bag contains a given entry.
         @param anEntry  the entry to locate
         @return true if this bag contains anEntry, or false otherwise */
   public boolean contains(T anEntry) 
    {
        // if getFrequencyOf(anEntry) returns a number greaters than zero, then the
        // entry is contained in the array
        return list.contains(anEntry);
    } // end contains
   
    /** Removes one unspecified entry from this bag, if possible.
       @return either the removed entry, if the removal
        was successful, or null. This method will remove the last
        entry*/
    public T remove()
    {        
        // temporal variable to hold the object to be removed
        T removed_object;
        //check if array is not empty
        if(!isEmpty()){
            removed_object = list.remove(list.size()-1); // retreive last item of array
        }else{
            removed_object = null; // no object can be removed.
        }
        
        return removed_object; // return result
    } // end remove
    
    /** Removes one occurrence of a given entry from this bag.
       @param anEntry  the entry to be removed
       @return true if the removal was successful, or false otherwise */
    public boolean remove(T anEntry) 
    {
        boolean result = true;
        // get position of the given entry, if it exist
        int position = list.indexOf(anEntry);
        // check if given entry exist at least one time in our array
        if(position!= -1){
            // if a ocurrence is find, then copy the last item of the array to position
            list.set(position, list.get(list.size()-1));
            // delete last item
            list.remove(list.size()-1);
        }else{
            result = false;
        }
        return result;
    } // end remove
     

    /** Removes all entries from this bag. */
    public void clear() 
    {
        // loop to delete items as long as the array contains entries
        while(!isEmpty()){
            list.remove(0); // remove an object at a time
        }
    
    } // end clear


    /** Retrieves all entries that are in this bag.
         @return a newly allocated array of all the entries in the bag */
    public Object[] toArray()
    {
        return list.toArray();
    } // end toArray
    
    /** Retrieves all entries that are in this bag.
         @return a array of type <T>[] all the entries in the bag */
    public T[] toArray(T[] result)
    {
        return list.toArray(result);
    } // end toArray
    
    /** Converts all the data in the bag into one big String
        @return a very long String of objects contained in bag */
    public String toString()
    {   
       // return list.toString().substring(1,list.size()+2);
        return list.toString();
    }    
    
    /**
     * Returns location of a specified entry in the array 
     * @param anEntry the entry to locate
     * @return the index where the specified entry is located
     */
    private int getIndexOf(T anEntry) 
    {
        return list.indexOf(anEntry);
    }
} // end Bag
